#!/usr/bin/env python

# (c) Copyright [2020 - 2021] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (Micro Focus) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

#functionality: Json file will be generated with health status of eDirectory and NCP File Server. 
		# Main service "eDirectory and NCP File Server" will be healthy only if both children "NCP File Server" and "eDirectory" are healthy. 
		# "ndsd" and "ncp2nss" services should be running for "NCP File Server" to be healthy.
		# "ndsd" and "nldap" services should be running for "eDirectory" to be healthy.

import os
import json
import fcntl
import commands
from time import strptime, mktime
from collections import OrderedDict

NCPandNDSJsonFile = "/var/opt/novell/ncp/ncpserv.json"

#Write the health of "eDirectory and NCP service" to /var/opt/novell/ncp/ncpserv.json
def writeJSON(dictionary):
	try:
		with open(NCPandNDSJsonFile, "w") as ncpstatusfile:
			fcntl.flock(ncpstatusfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
			json.dump(dictionary, ncpstatusfile, indent=4, separators=(", ", ": "))
			fcntl.flock(ncpstatusfile, fcntl.LOCK_UN)
	except IOError:
		raise Exception("Failed to open/write the Json file", NCPandNDSJsonFile)

def getHealthStatus():
	ndsdsystemctl_cmd = "systemctl is-active ndsd"
	ndsdstatus = commands.getoutput(ndsdsystemctl_cmd)
	ncp2nss_systemctlcmd = "systemctl is-active ncp2nss"
	ncp2nss_status = commands.getoutput(ncp2nss_systemctlcmd)
	jsonncp_stream = OrderedDict(
		[
			('serviceName', 'eDirectory and NCP Server'),
			('status',''),
			('AdditionalInfo', ''),
			('children',
				[OrderedDict(
				[
					('serviceName', 'NCP Server'),
					('status', ''),
					('AdditionalInfo', ''),
					('children',
						[OrderedDict(
						[
							('serviceName', 'ndsd'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
						]
						),
						OrderedDict(
						[
							('serviceName', 'ncp2nss'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
						])]
					)
				]),
				OrderedDict(
				[
					('serviceName', 'eDirectory'),
					('status', ''),
					('AdditionalInfo', ''),
					('children',
						[OrderedDict(
						[
							('serviceName', 'ndsd'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
						]
						),
						OrderedDict(
						[
							('serviceName', 'LDAP Services for eDirectory'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
						])]
					)
				])]
			)
		])
	ldap_daemon,ldapstatus_output = commands.getstatusoutput("/opt/novell/eDirectory/sbin/nldap -s")

	if(ndsdstatus == "inactive"):
		jsonncp_stream["children"][0]["children"][0]["status"] = ("UnHealthy")
		jsonncp_stream["children"][0]["children"][0]["runningSince"] = 0
		jsonncp_stream["children"][0]["children"][0]["AdditionalInfo"] = ["ndsd service","Common_Inactive","Common_ResolveService"]
		jsonncp_stream["children"][1]["children"][0]["status"] = ("UnHealthy")
		jsonncp_stream["children"][1]["children"][0]["runningSince"] = 0
		jsonncp_stream["children"][1]["children"][0]["AdditionalInfo"] = ["ndsd service","Common_Inactive","Common_ResolveService"]

	else:
		ndsdtimestamp_cmd = "systemctl show -p ExecMainStartTimestamp ndsd"
		ndsdtimestatus = commands.getoutput(ndsdtimestamp_cmd)
		ndsdtimestamp_output = ndsdtimestatus.strip().split("=")
		ndsdtarget_timestamp = strptime(ndsdtimestamp_output[1], '%a %Y-%m-%d %H:%M:%S %Z')
		ndsdepoch_time = mktime(ndsdtarget_timestamp)
		jsonncp_stream["children"][0]["children"][0]["status"] = ("Healthy")
		jsonncp_stream["children"][0]["children"][0]["runningSince"] = int(ndsdepoch_time)
		jsonncp_stream["children"][1]["children"][0]["status"] = ("Healthy")
		jsonncp_stream["children"][1]["children"][0]["runningSince"] = int(ndsdepoch_time)

	if(ncp2nss_status == "inactive"):
		jsonncp_stream["children"][0]["children"][1]["status"] = ("UnHealthy")
		jsonncp_stream["children"][0]["children"][1]["runningSince"] = 0
		jsonncp_stream["children"][0]["children"][1]["AdditionalInfo"] = ["ncp2nss service","Common_Inactive","Common_ResolveService"]
	else:
		ncp2nsstimestamp_cmd = "systemctl show -p ExecMainStartTimestamp ncp2nss"
		ncp2nsstimestatus = commands.getoutput(ncp2nsstimestamp_cmd)
		ncp2nsstimestamp_output= ncp2nsstimestatus.strip().split("=")
		ncp2nsstarget_timestamp = strptime(ncp2nsstimestamp_output[1], '%a %Y-%m-%d %H:%M:%S %Z')
		ncp2nssepoch_time = mktime(ncp2nsstarget_timestamp)
		jsonncp_stream["children"][0]["children"][1]["status"] = ("Healthy")
		jsonncp_stream["children"][0]["children"][1]["runningSince"] = int(ncp2nssepoch_time)

	if (ldap_daemon == 0):
		jsonncp_stream["children"][1]["children"][1]["status"] = ("Healthy")
		if(ndsdstatus == "active"):
			jsonncp_stream["children"][1]["children"][1]["runningSince"] = int(ndsdepoch_time)
		else:
			jsonncp_stream["children"][1]["children"][1]["runningSince"] = 0
	else:
		jsonncp_stream["children"][1]["children"][1]["status"] = ("UnHealthy")
		jsonncp_stream["children"][1]["children"][1]["runningSince"] = 0
		jsonncp_stream["children"][1]["children"][1]["AdditionalInfo"] = ["LDAP Services for eDirectory","Common_Inactives","Common_ResolveServices"]

	if (ndsdstatus == 'active') and (ncp2nss_status == 'active'):
		jsonncp_stream["children"][0]["status"] = ("Healthy")
	else:
		jsonncp_stream["children"][0]["status"] = ("UnHealthy")
		jsonncp_stream["children"][0]["AdditionalInfo"] = ["Common_DependentService","NCP","Common_Inactives","Common_ResolveServices"]
	if (ndsdstatus == 'active') and (ldap_daemon == 0):
		jsonncp_stream["children"][1]["status"] = ("Healthy")
	else:
		jsonncp_stream["children"][1]["status"] = ("UnHealthy")
		jsonncp_stream["children"][1]["AdditionalInfo"] = ["Common_DependentService","eDirectory","Common_Inactives","Common_ResolveServices"]

	if ((jsonncp_stream["children"][0]["status"] == "Healthy") and (jsonncp_stream["children"][1]["status"] == "Healthy")):
		jsonncp_stream["status"] = ("Healthy")
	else:
		jsonncp_stream["status"] = ("UnHealthy")
		jsonncp_stream["AdditionalInfo"] = ["Common_DependentService","eDirectoryandNCP_Unhealthy","Common_ResolveServices"]
	return jsonncp_stream

healthData = getHealthStatus()

try:
	writeJSON(healthData)
except Exception:
	exit(1)

